// Scenesaver: copyright 2004 Sam Stafford.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32
	#include <direct.h>
	extern int Width, Height;
#endif

#include "globals.h"

#define LTG_ABOUT "\
# Scenesaver - the best slideshow screensaver ever\n\
# from http://www.leadtogold.com\n\
# \n\
# \n\
# \n\
# Your Scenesaver directory is:\n\
"

#define LTG_INSTR "\
# To specify the images that Scenesaver will use, create a file\n\
# called Scenes.txt in this directory, and list the images, one\n\
# file per line (no wildcards).  The images will be used in the\n\
# order listed.  It is recommended that all of the images be the\n\
# same dimensions; if they are not, the dimensions of the first\n\
# image will be used and all subsequent images will be clipped to\n\
# those dimensions.\n\
# \n\
# When screensaver is running, push 'c' to switch camera mode.\n\
# When in \"chase-cam\" mode, push 'v' to switch viewed creature.\n\
# Push 'b' to switch display buffer.\n\
# Push 'n' to switch to a new image.\n\
# Push 'x' to toggle creatures (in)visible.\n\
# Push 'z' to dump a screenshot of the image into this directory.\n\
# \n\
"

void loadsettings();
void savesettings();

int config()
{
	loadsettings();
	savesettings();
#ifdef WIN32
	return system( "notepad Scenesaver.cfg" );
#else
	return system( "vi Scenesaver.cfg" );
#endif
};

void loadsettings()
{
	settings.save = true;
	settings.mutate = 9;
	settings.digest = 30;
	settings.camera = 'w';
	settings.visible = true;
	settings.radius = 5;
	settings.vision = 60;
	settings.fitness = 500;
	settings.image = 5000;
	settings.diff = 80;
	settings.pop = 10;
	settings.autoscale = true;
	settings.shuffle = false;

	FILE* f = fopen( "Scenesaver.cfg", "r" );
	if ( !f )
	{
		savesettings();
		return;
	}
	char c[64];
	while ( fgets( c, 64, f ) )
	{
		if ( !memcmp( c, "savegenes=T", 11 ) )
			settings.save = true;
		else if ( !memcmp( c, "savegenes=F", 11 ) )
			settings.save = false;
		else if ( !memcmp( c, "setcamera=W", 11 ) )
			settings.camera = 'w';
		else if ( !memcmp( c, "setcamera=C", 11 ) )
			settings.camera = 'c';
		else if ( !memcmp( c, "drawcrtrs=T", 11 ) )
			settings.visible = true;
		else if ( !memcmp( c, "drawcrtrs=F", 11 ) )
			settings.visible = false;
		else if ( !memcmp( c, "autoscale=T", 11 ) )
			settings.autoscale = true;
		else if ( !memcmp( c, "autoscale=F", 11 ) )
			settings.autoscale = false;
		else if ( !memcmp( c, "randomize=T", 11 ) )
			settings.shuffle = true;
		else if ( !memcmp( c, "randomize=F", 11 ) )
			settings.shuffle = false;
		else if ( !memcmp( c, "mutations=", 10 ) )
			settings.mutate = atoi( c+10 );
		else if ( !memcmp( c, "digestion=", 10 ) )
			settings.digest = atoi( c+10 );
		else if ( !memcmp( c, "maxradius=", 10 ) )
			settings.radius = atoi( c+10 );
		else if ( !memcmp( c, "visionrng=", 10 ) )
			settings.vision = atoi( c+10 );
		else if ( !memcmp( c, "t_fitness=", 10 ) )
			settings.fitness = atoi( c+10 );
		else if ( !memcmp( c, "t_nwimage=", 10 ) )
			settings.image = atoi( c+10 );
		else if ( !memcmp( c, "pctchange=", 10 ) )
			settings.diff = atoi( c+10 );
		else if ( !memcmp( c, "creatures=", 10 ) )
			settings.pop = atoi( c+10 );
	}

	fclose( f );

	//Sanity checks
	if ( settings.mutate < 0 ) settings.mutate = 9;
	if ( settings.digest < 0 ) settings.digest = 30;
	if ( settings.radius < 0 ) settings.radius = 5;
	if ( settings.vision < 0 ) settings.vision = 30;
	if ( settings.fitness < 0 ) settings.fitness = 500;
	if ( settings.image < 0 ) settings.fitness = 5000;
	if ( settings.diff < 0 || settings.diff > 100 ) settings.diff = 50;
	if ( settings.pop < 0 ) settings.pop = 10;
}

void savesettings()
{
	FILE* f = fopen( "Scenesaver.cfg", "w" );
	if ( !f ) return;

	char c;

	fprintf( f, LTG_ABOUT );
	fprintf( f, "# %s\n#\n", getcwd( 0x0, 0 ) );
	fprintf( f, LTG_INSTR );

	fprintf( f, "#\n#\n# SETTINGS\n#\n" );

	fprintf( f, "#\n# Simulation settings\n#\n" );

	fprintf( f, "# Number of animals in the world?\n" );
	fprintf( f, "creatures=%d\n", settings.pop );

	fprintf( f, "# Save genotypes on exit?\n" );
	c = settings.save ? 'T' : 'F';
	fprintf( f, "savegenes=%c\n", c );

	fprintf( f, "# Number of timesteps per fitness cycle?\n" );
	fprintf( f, "t_fitness=%d\n", settings.fitness );

	fprintf( f, "# Number of mutations per fitness cycle?\n" );
	fprintf( f, "mutations=%d\n", settings.mutate );

	fprintf( f, "# Number of timesteps before automatic image swap?\n" );
	fprintf( f, "t_nwimage=%d\n", settings.image );

	fprintf( f, "# Percent of image changes before automatic image swap?\n" );
	fprintf( f, "pctchange=%d\n", settings.diff );

	fprintf( f, "# Randomized image order instead of sequential?\n" );
	c = settings.shuffle ? 'T' : 'F';
	fprintf( f, "randomize=%c\n", c );

	fprintf( f, "# Amount of color each animal can change per step?\n" );
	fprintf( f, "digestion=%d\n", settings.digest );

	fprintf( f, "# Maximum radius, in pixels, of each animal?\n" );
	fprintf( f, "maxradius=%d\n", settings.radius );

	fprintf( f, "# Vision range, in pixels, for each animal?\n" );
	fprintf( f, "visionrng=%d\n", settings.vision );

	fprintf( f, "#\n# View settings\n#\n" );

	fprintf( f, "# Default camera setting?  W for world, C for chase\n" );
	if ( settings.camera == 'c' ) c = 'C';
	else c = 'W';
	fprintf( f, "setcamera=%c\n", c );

	fprintf( f, "# Draw creatures in world view?\n" );
	c = settings.visible ? 'T' : 'F';
	fprintf( f, "drawcrtrs=%c\n", c );

	fprintf( f, "# Auto-scale images instead of clip?\n" );
	c = settings.autoscale ? 'T' : 'F';
	fprintf( f, "autoscale=%c\n", c );

	fclose( f );
}

void loadimages( int* argc, char* ** argv )
{
	//initialize argc and argv to checkable error values
	*argc = 0;
	*argv = new char*; //yes, I leak a byte here.  Byte me.  Har har har.
	*argv[0] = 0;
	FILE* f = fopen( "Scenes.txt", "r" );
	if ( !f )
	{
		return;
	}
	char c[1024];
	int lines = 0;
	while ( fgets( c, 1024, f ) )
	{
		lines++;
	}
	*argc = lines;
	rewind( f );

	*argv = new char*[lines];
	int i = 0;
	while ( fgets( c, 1024, f ) )
	{
		(*argv)[i] = new char[strlen(c)];
		strncpy( (*argv)[i], c, strlen(c)-1 );
		(*argv)[i][strlen(c)-1] = 0;
		i++;
	}
}

